
-- ***********************************************************************************************
-- IMPORTANT: If you use this extension, you must delete the talk, talk_to, and talk_to_a verbs 
-- from the standard library VERBS.i file to avoid getting an error from having two verbs 
-- with the same name.
-- ***********************************************************************************************

--==================================================================================
-- The Conversation system adds the TALKERS attribute to the THING class. When a THING has an 
-- actor ID in its TALKERS attribute, the name of that thing will appear in the list of topics 
-- that the reader/player can talk to the actor about. You can include multiple IDs when more 
-- than one Actor can talk about the same topic.
--==================================================================================
Add to every thing 
	Has talkers {thing}.	-- Used by the T verb.
End add to.

--==================================================================================
-- The Conversation system adds the attributes to the ACTOR class that it needs in order
-- for the reader/player to carry on a conversation with an NPC.
--==================================================================================
Add to every actor	

	-- As mentioned in the ADVICE/HINT SUBSYSTEM comments that appear later, 
	-- it is possible for the conversation extension to allow the NPCs in your game to give 
	-- advice and hints to the reader/player when he or she gets stuck. The basic idea is that 
	-- the reader/player can go to the nearest NPC to get help.

	-- Use the provide_help attribute in the actor instance to turn the help capability on or off 
	-- for any particular actor. If you set the attribute value here, it will globally turn the
	-- the help capability on or off for every actor in the game.
		Can not provide_help.	-- Turns advice/hint capability off
	--	Can provide_help.	  	-- Turns advice/hint capability on

	--==================================================================================	
	-- While these next three attributes appear in every actor instance, the conversation 
	-- extension uses the values in the hero instance and ignores them in all other actors.
	--==================================================================================
	Has listener hero.  -- Contains the actor ID of the target NPC of the current 
						-- conversation.
						-- Hero is used as the intial value of the attribute to initialize
						-- it with the actor class to avoid creating an abstract attribute.
						-- Consequently, the hero is listening to himself or herself 
						-- talk at the start of the game. However, the talk_to verb 
						-- sets it to a new value during course of the game each time 
						-- the reader/player talks to a different NPC.

	Has targetActor hero. -- Contains the ID of the target NPC for internal use by the 
						  -- conversation system. Hero is used as the intial value of the 
						  -- attribute to initialize it with the actor class to avoid 
						  -- creating an abstract attribute.

	Has topic_flag 0.	-- Used to control the topic listing by the conversation system.

	--==================================================================================
	-- The conversation extension assumes that you will want to have each actor to give 
	-- a unique header for the topics listing to the first time the reader/player talkes to 
	-- an actor, and then have a different response for all other times. However, you can give 
	-- both of these attributes the same value if you want to.

	-- Normally, you would initialize these two attributes at the start of the game, but you 
	-- may alter their values at any time you deem it necessary during the course of the game.	

    Has topics_header1 "Header1".	-- NPC header for topic list for talk_to the first time  
	Has topics_header2 "Header2".	-- NPC header for topic list for talk_to all other times
	--==================================================================================
	-- The Conversation extension must have a response for when the reader/player attempts 
	-- to talk about a topic that is not on the topics list. 
	-- It uses the unknown_topic_header from whatever NPC the reader/player is talking to 
	-- as the header for the topic list it displays to remind the reader/player what topics 
	-- are available.

	-- Use the default value given here for all actors or modify it to your preference.
	-- If you modify it at the instance level, you will affect only that particular actor's 
	-- response.
    Has unknown_topic_header "<NPC_name> gives you a blank look. After a 
    rethinking the issue for a moment, you realize that all you really want to talk about is:".
	--==================================================================================
	-- You may want to have a NPC in your game that cannot talk, an animal for example.
 
	Is not mute. --  All actors are initialized as NOT MUTE by default.

	-- The default response for an actor that is mute when the reader/player attempts 
	-- to initiate a conversation. You may edit this response as you see fit.

	Has mute_response "This character seems to be mute.".
	--==================================================================================
End add to.

--==================================================================================
-- Every object in the game, including Actors, can be a topic in a conversation. However,
-- most games will require many conversation topics that are not actual objects in the game.
-- Since the t verb can refer only to existing objects, a new class of objects is needed 
-- just for topic objects.
--==================================================================================
Every topic isa thing -- new class for regular topic objects
End every.

The topics_dummy isa topic 	-- Internal use by the T verb
	Name topics
End the.

--============================
-- EXAMPLE of Topic Object 
--============================

--The fly_dummy isa topic -- sample topic object
--	Name fly
--	Has talkers {janet, adriana}. 	-- This topic will appear in list of topics for any actor 
									-- you add to its talkers attribute.
									-- While you can initialize the set with names (as shown
									-- here) to give the topic to one or more actors at the 
									-- start of the game, you would normally add names with 
									-- include statements during the course of the game. 
									-- Example: INCLUDE JANET IN TALKERS OF FLY_DUMMY.
--	Verb t does
		-- in this example, both Janet and Adriana will list 'fly' in their topic list
		-- when the reader/player uses the talk_to verb. Example: TALK TO JANET
--		Depending on listener of hero
--			= janet then
--				"Oh no! A fly got in here."
--			= adriana then
--				"Kill the fly!."
--		End depend.
--	End verb.
--End the. 

--==================================================================================
-- NOTE: When the topic list for an actor is empty, the conversation system displays
-- (none) instead of a blank list. Since the reader/player may attempt to talk
-- about the 'none' topic, the system needs a default response that will clue the 
-- reader/player as to what is happening. 
--==================================================================================
	The none_dummy isa topic -- required for default response to t none command
		Name none
		Verb t does only
			"The (none) under either TOPICS or ADVICE/HINT means that the list is empty at this time."
		End verb.
	End the.

--==================================================================================
--		START OF ADVICE/HINT SUBSYSTEM
--==================================================================================
	-- It is possible to use the conversation system to have the NPCs in your game give 
	-- advice and hints to the reader/player when he or she gets stuck. The basic idea is that 
	-- the reader/player can go to the nearest NPC to get help. This capability is 
	-- intended to be a two-tiered system where advice is a non-spoiler nudge in the right 
	-- direction, and a hint is a spoiler that tells the player what to do but does not list 
	-- the actual commands needed. (Save that for the walkthrough.) The 
	-- conversation extension will list hint and advice topics at the bottom of the topics list 
	-- that appears to the player. 

	-- If you decide to use this capability, you can set the provide_help attribute to true
	-- for each actor instance that can provide help. Example:
	--
	-- 		The Grassi Isa male at stone_room 
	--			Name Grassi Name old man Name old
	-- 			Can provide_help.
	--			...
	--			...
	--		End the.
	--
	-- If you want to globally enable all actors in your game, you can set the value 
	-- of the provide_help attribute to true in this extension where the attribute was defined 
	-- and added to every actor.
	--
	-- The advice_text and the hint_text attributes in each NPC hold the text for the current 
	-- help that particular NPC can give to the reader/player.

	-- To use the conversation extension's' help capability,your code has to update the 
	-- advice_text and/or help/text attributes of the appropriate NPC as the reader/player 
	-- moves through your game. Also your code must include that NPC in the talkers of the 
	-- appropriate dummy. For example:
	--
	--  		Set advice_text of forest_fairy to
	--				"The fairy flies over to the dumpster, hovers over it, and points down.".
	--			Include forest_fairy in talkers of advice_dummy.
	---------------------------------------------------------------------------------------

		Add to every actor 
			Has advice_text "".   	-- Contains advice text that is not a spoiler
			Has hint_text "".	  	-- Contains hint text that is a spoiler
		End add to.
					
		Every helper isa thing-- new class for the advice/hint topic objects
		End every.
		
		The advice_dummy Isa helper 	-- used by all actors
			Name Advice 
			Verb t does only
				Say advice_text of listener of hero.
				Exclude listener of hero from talkers of subj.
			End verb.
		End the.
		
		The hint_dummy Isa helper 	-- used by all actors
			Name Hint
			Verb t does only
				Say hint_text of listener of hero.
				Exclude listener of hero from talkers of subj.
			End verb.
		End the.
--==================================================================================
--		END OF ADVICE/HINT SUBSYSTEM
--==================================================================================


--==================================================================================
-- The Conversation extension defines these verbs to be the commands that the reader/player 
-- to carry on a conversation:
--==================================================================================
	SYNTAX talk_to = talk 'to' (act)! 
		WHERE act ISA ACTOR
	 		ELSE 
				IF act IS NOT plural then
					"That's not"
				  ELSE "Those are not"
				END IF.
					"something you can talk to."
	
	ADD TO EVERY ACTOR 
		Has topic_count 0.
		Has help_topic_count 0.
	
		Has t_flag 1.	-- topics_header1 or topics_header2
			
		VERB talk_to -- the target actor
			Check act here -- verify that the target actor is present
				Else Say act. "is too far away to talk to."
			AND act is reachable
				Else Say act. "can't hear you."
			AND act not = hero
				Else 
					"You are now listening to yourself talk."
					Set listener of hero to hero.
			DOES
				Set targetActor of hero to act.
				Schedule talking at hero after 0.
		End verb.
	End Add.

	--==================================================================================
	SYNTAX tt = tt (act)! 
		WHERE act ISA ACTOR
	 		ELSE 
				IF act IS NOT plural then
					"That's not"
				  ELSE "Those are not"
				END IF.
					"something you can talk to."
	
	Syntax talk = talk (act)!
		WHERE act ISA ACTOR
	 		ELSE 
				IF act IS NOT plural then
					"That's not"
				  ELSE "Those are not"
				END IF.
					"something you can talk to."
	
	ADD TO EVERY ACTOR 
		VERB tt, talk -- the target actor
			Check act here -- verify that the target actor is present
				Else Say act. "is too far away to talk to."
			AND act is reachable
				Else Say act. "can't hear you."
			AND act not = hero
				Else 
					"You are now listening to yourself talk."
					Set listener of hero to hero.
			DOES
				Set targetActor of hero to act.
				Schedule talking at hero after 0.
		End verb.
	End Add.
	
	--==================================================================================
	SYNTAX t0 = t.
	
	VERB t0
		DOES
			"You're presently talking to " Say listener of hero. "."
			"$n"
			Schedule list_topics at hero after 0.		
	END VERB.
	
	--==================================================================================
	Syntax topics = topics.
	
	Synonyms topic = topics.
	
	Verb topics
		Check listener of hero <> hero
			Else "You're talking to yourself."
		AND listener of hero at current location
			Else Say listener of hero. " is too far away for conversation."
		Does
				"You're presently talking to " Say listener of hero. "."
				"$n"
				Schedule list_topics at hero after 0.
	End verb.
	
	--==================================================================================
	SYNTAX t = t (subj)! 
		WHERE subj ISA thing			-- includes both topic objects and actors
	     	ELSE 
				IF subj IS NOT plural Then
					"That's not"
				  ELSE "Those are not"
				END IF.
					"a topic that you can talk about with "
					Say listener of hero. "."
					"[You probably didn't get the wording of the topic right, so try again exactly as listed. 
					Or, you can try using just the last word in the topic. Finally, you could try talking to 
					someone 
					else.]"
	
	ADD TO EVERY thing 
	  VERB t 
		Check listener of hero at current location
			Else "You take a quick look around and realize that " 
					Say listener of hero. "isn't here."
		AND listener of hero <> hero
			Else "The moment you realize that you're just talking to yourself, you remember that you have to talk to 
			someone before you try to talk about something."
				
		AND subj <> topics_dummy  -- response for t topics
			Else
				Set topic_count of listener of hero to 0.
				For each t Isa thing Do  	-- looks through all things (includes objects, topics, and actors) to
											-- count the topics
					If listener of hero in talkers of t then -- count listener's current topics
						Increase topic_count of listener of hero.
					End if.
				End for each.
				If topic_count of listener of hero = 0 then -- no topics
					"As you think about what you'd like to talk to" 
					Say listener of hero.
					"about, nothing comes to mind."
				  Else -- list the topics
					"You do a quick mental review and find that you might like to talk to "
					Say listener of hero.
					"about:"
					"$n"
					Schedule list_topics at hero after 0.
				End if.	
		AND listener of hero in talkers of this 
			Else 					-- for topic not on topic list display list of topics
				Set topic_count of listener of hero to 0.
				For each t Isa thing Do  -- looks through all things (includes t_objects and actors)
					If listener of hero in talkers of t then -- count listener's current topics
						Increase topic_count of listener of hero.
					End if.
				End for each.
				If topic_count of listener of hero = 0 Then -- empty list response
					"On second thought, you realize that you really didn't want to talk to "
					Say listener of hero.
					"about anything at this time."
				  Else
					Say unknown_topic_header of listener of hero. -- repeat the list
					"$n"
					Schedule list_topics at hero after 0.
				End if.
		Does
			-- NOTE: You must override the T verb in the instance (object, topic object, or actor) to display 
			-- a textual response for the <topic> when the reader/player uses the T <topic> command.	
			-- For example:
				--	Verb t does
				--		Depending on listener of hero
				--			= janet then
				--				"Oh no! A fly got in here."
				--			= adriana then
				--				"Kill the fly!."
				--		End depend.
				--	End verb.

			Exclude listener of hero from talkers of subj. 	-- automatically removes the <topic> from the 
															-- topic list for the current conversation so
															-- the reader/player will see a topic list that
															-- gets shorter and shorter as he or she talks 
															-- to the NPC.
	  END VERB.
	END ADD TO.

-- ======================================================================================================
-- These events are used by the conversation system as functions to display the list of topics available
-- for the current conversation. You should modify the text strings if you wish to change the default
-- responses.
-- ======================================================================================================
	Event talking
				If listener of hero = targetActor of hero
					Then
						Say listener of hero. "is already talking to you."
						"$n"
						Schedule list_topics at hero after 0.
				Elsif targetActor of hero is mute
					Then
						Say mute_response of targetActor of hero.
				Else					
					Set listener of hero to targetActor of hero.	-- sets the listener to the current actor
					If listener of hero at current location 
						Then -- added because a timed EVENT could remove the actor
							 -- in a rare case where the TALK TO command triggers the event 
							Set topic_count of listener of hero to 0.
							For each t Isa thing Do				-- looks through all things 
																-- (includes actors and t_objects)
								If listener of hero in talkers of t then 	-- count listener's current topics
									Increase topic_count of listener of hero.
								End if.
							End for each.
							If topic_count of listener of hero = 0 
								Then		-- no topics currently available
									"As soon as you get " Say listener of hero. "to pay attention to you, you find 
									that you actually don't have anything you wish to talk about." 	
								Elsif help_topic_count of listener of hero > 0
									Then
										"$nYou can't think of any thing that you need to talk about at this time."
								Else
									If t_flag of listener of hero = 1 
										Then 
											Say topics_header1 of listener of hero. -- 1st time with this NPC
											Set t_flag of listener of hero to 2.
										Else
											Say topics_header2 of listener of hero. -- at other times with this NPC
									End if.
									Schedule list_topics at hero after 0.	
									"$n"		
							End if.
						Else
							"$p"Say listener of hero. "is no longer present."
					End if.
				End if.
	End event.
		-- ======================================================================================================
	Event list_topics
		"$n-- (topics) --"
		Set topic_flag of hero to 0.
		Exclude listener of hero from talkers of none_dummy.
	
		For each t Isa actor Do -- looks through all actors to list them first
			If listener of hero in talkers of t then 
				Increase topic_flag of hero.
				"$i" Say t.   		-- displays a single list entry
			End if.
		End for each.
		For each t Isa topic Do -- looks through all topic objects to list them
			If listener of hero in talkers of t then 
				Increase topic_flag of hero.
				"$i" Say t.   		-- displays a single list entry
			End if.
		End for each.
		For each t Isa object Do -- looks through all standard objects to list them next
			If listener of hero in talkers of t then 
				Increase topic_flag of hero.
				"$i" Say t.   		-- displays a single list entry
			End if.
		End for each.
		If topic_flag of hero = 0
			Then
				"$i(none)"
				Include listener of hero in talkers of none_dummy.
		End if.
	
		-- List the advice and hint topics at the bottom of the topics listing
		-- NOTICE that the IF clause will enable or disable the advice and hint topics list
		-- depending on the value of the provide_help attribute in the target NPC instance.
		If listener of hero can provide_help
			Then
				"$p-- (advice/hint) --" -- separator
					Set topic_count of listener of hero to 0.
				For each t Isa helper Do -- looks through all help objects to list them last
					If listener of hero in talkers of t then 
						Increase topic_count of listener of hero.
						"$i" Say t.   -- displays a single list entry
					End if.	
				End for each.
				If topic_count of listener of hero = 0
					Then
						"$i(none)"
						Include listener of hero in talkers of none_dummy.
				End if.	
		End if.
		-------end of advice and hint statements------------------------------------------------	
	End event.

